# Higher Lower Game - version 1
# Usability testing version.  Includes assembled components.

# To Do
# Allow users to press <enter> for default values of 1, 100, 3 rounds
# For first guess, ask user to guess a number between bounds, then give higher / lower hint

import math
import random

# Functions go here...


# Number checking function goes here
def intcheck(question, low=None, high=None):

    while True:

        # sets up error messages
        if low is not None and high is not None:
            error = "Please enter an integer between {} and {} (inclusive)".format(low, high)
        elif low is not None and high is None:
            error = "Please enter an integer that is more than or equal to {}".format(low)
        elif low is None and high is not None:
            error = "Please enter an integer that is less than or equal to {}".format(high)
        else:
            error = "Please enter an integer"

        try:
            response = int(input(question))

            # Checks response is not too low
            if low is not None and response < low:
                print(error)
                continue

            # Checks response is not too high
            if high is not None and response > high:
                print(error)
                continue

            return response

        # checks input is a integer
        except ValueError:
            print(error)
            continue


def hl_statement(statement, char):
    print()
    print(char*len(statement))
    print(statement)
    print(char*len(statement))
    print

# Main Routine is here...

# welcom message - displayed once only
print("**** Welcome to the Higher Lower Game ****")
print()
print("For each game you will be asked to... \n"
      "- Enter a 'low' and 'high' number.  The computer will randomly generate a\n"
      " 'secret' number between your two chosen numbers.  It will use these numbers for all \n"
      "the rounds in a given game.\n"
      "- The computer will calculate how many guesses you are allowed \n"
      "- enter the number of rounds you want to play \n"
      "- guess the secret number \n \n"

      "Good Luck!")
print()
input("Please press <enter> to begin...")
print()

keep_going = ""
while keep_going == "":

    # Get User input ...
    lowest = intcheck("Low Number: ")
    highest = intcheck("High Number: ", lowest + 1)
    rounds = intcheck("Rounds: ", 1)
    print()

    # Calculate maximum # of guesses...
    range = highest - lowest + 1
    max_raw = math.log2(range)  # finds maximum # of guesses using binary search
    max_upped = math.ceil(max_raw)  # rounds up (ceil --> ceiling)
    max_guesses = max_upped + 1

    # initialise values for each game...
    num_won = 0
    rounds_played = 0
    game_stats=[]

    while rounds_played < rounds:

        # initialise values for each round...

        already_guessed = []    # list to store guesses / prevent duplicates
        guess = ""
        guesses_left = max_guesses

        # Generate secret number
        secret = random.randint(lowest, highest)
        # print("The secret number is {}".format(secret))

        print()
        hl_statement("Round {} of {} - you have {} guesses left...".format(rounds_played+1, rounds, guesses_left), "-")

        while guess != secret and guesses_left >= 1:

            guess = intcheck("Guess: ", lowest, highest)
            # checks that guess is not a duplicate
            if guess in already_guessed:
                hl_statement("You already guessed that number!  Please try again.  "
                      "Guesses left: {}".format(guesses_left), "!")
                continue

            guesses_left -= 1
            already_guessed.append(guess)

            # more than one guess left, feedback suggests trying again...
            if guesses_left >= 1:

                if guess < secret:
                    feedback = "Too low, try a higher number.  Guesses left: {}".format(guesses_left)
                    symbol = "^"

                elif guess > secret:
                   feedback = "Too high, try a lower number.  Guesses left: {}".format(guesses_left)
                   symbol = "v"

                elif guess == secret:
                    if guesses_left == max_guesses - 1:
                        feedback = "Amazing!  You got it in one guess"
                    else:
                       feedback = "Well done, you got it in {} guesses".format(max_guesses - guesses_left)
                    num_won += 1
                    symbol = "*"

            # One guess left, feedback does not suggest trying another number
            else:
                if guess < secret:
                    feedback = "Too low! You are out of guesses :( "
                    symbol = "^"
                    guesses_left -= 1   # penalty point for losing
                elif guess > secret:
                    feedback = "Too high.  You are out of guesses :( "
                    symbol = "v"
                    guesses_left -= 1   # penalty point for losing
                else:
                    feedback = "Lucky!  You almost ran out of guesses :)"
                    symbol = "*"
                    num_won += 1

            hl_statement(feedback, symbol)


        game_stats.append(max_guesses - guesses_left)
        print()
        hl_statement("Won: {} \t | \t Lost: {}".format(num_won, rounds_played - num_won + 1), "-")
        rounds_played += 1

    input("Please press enter to see your game summary / statistics")

    # print each round's outcome...
    print()
    print("*** Game Scores***")
    list_count = 1
    for item in game_stats:

        # indicates if game has been won or lost
        if item > max_guesses:
            status = "lost, ran out of guesses"
        else:
            status = "won"

        print("Round {}: {} ({})".format(list_count, item, status))
        list_count += 1

    # Calculate (and then print) game statistics
    game_stats.sort()
    best = game_stats[0]    # first item in sorted list
    worst = game_stats[-1]  # last item in sorted list
    average = sum(game_stats)/len(game_stats)

    print()
    print("*** Summary Statistics ***")
    print("Best: {}".format(best))
    print("Worst: {}".format(worst))
    print("Average: {:.2f}".format(average))

    print()
    keep_going = input("Press <enter> to play again or any key to quit: ")
    print()

print("Thank you for playing.  Good bye")
