# Car Racer Version 2 - Complete Game (post usability testing)
# Includes clearer headings and extra explanations


import random
import unicodedata as ud    # module for emoticons...

# Functions go here...

# function below races cars.  Previous winners populates a list which prevents cars which have
# won / placed from continuing to race.  Status is the placing (win, 1st or 2nd) and rolls
# is needed to keep track of the 'time'.  Both previous winners and rolls are returned
# as a list so that these variables can be passed back into the function for the 2nd and 3rd places.
def race_them(competitors, num_dice, previous_winners, status, rolls):

    # initialise local variables
    longest = ""
    place_winners = previous_winners

    while len(longest) < distance + len(name):
        if num_dice == 2:
            dice_throw = random.randint(1,6) + random.randint(1,6)            # rolls two dice
        else:
            dice_throw = random.randint(1,6)        # rolls one die

        if dice_throw not in place_winners:
            competitors[dice_throw - 1] += "0"   # add '0' to show car progress
            rolls += 1      # count dice roll only if it is not in place_winner list
        longest = max(all_cars, key=len)            # update longest so while loop ends

        place = longest[5:7]

    used = int(place)
    place_winners.append(used)  # add place winners to list so car is out of race

    # Put in place...
    to_label = competitors.index(longest)
    competitors[to_label] = "car #{} | {}".format(place, status)

    # print diagram (shows Placing)
    for item in competitors:
        print(item, sep="")

    # print win statement
    print()
    print("{}: car #{} \t Time: {} throws".format(status, place, rolls))

    # Return rolls
    return [place_winners, rolls, place]   # use a list to return more than one piece of data!

# function below checks that user input is an integer
def intcheck(question, low=None, high=None):

    # sets up error messages
    if low is not None and high is not None:
            error = "Please enter an integer between {} and {} " \
                    "(inclusive)".format(low, high)
    elif low is not None and high is None:
        error = "Please enter an integer that is more than or " \
                "equal to {}".format(low)
    elif low is None and high is not None:
        error = "Please enter an integer that is less than or " \
                "equal to {}".format(high)
    else:
        error = "Please enter an integer"

    while True:

        try:
            response = int(input(question))

            # Checks response is not too low
            if low is not None and response < low:
                print(error)
                continue

            # Checks response is not too high
            if high is not None and response > high:
                print(error)
                continue

            return response

        except ValueError:
            print(error)
            continue

# Function below formats statements / start of game etc clearly
def gen_statement(statement, char):
    print()
    print(char*len(statement))
    print(statement)
    print(char*len(statement))
    print


# Main Routine...

# Introduction and instructions (appears only once)
print("**** Welcome *****")
print()
print("To play you need to...\n"
      "- Choose a distance (between 10 and 25)\n"
      "- Choose the number of rounds \n"
      "- Choose how many dice to use <1 or 2>\n"
      "- Choose a car\n\n"
      "Scoring is as follows:\n"
      "- Win: 5 points\n"
      "- Second: 3 points\n"
      "- Third: 1 point\n\n"
      "Good Luck!")

games_played = 0    # used to show extra instructions only once

keep_going = ""
while keep_going == "":

    if games_played == 0:
        gen_statement(" -----  Let's Play -----", "-")
    else:
        gen_statement(" ----- New Game Starting ... -----", "-")

    # variables set up for each game
    game_summary = []
    rounds_played = 0
    score = 0
    distance = intcheck("How long should the race be <min 10, max 25>? ", 10, 25)
    num_rounds = intcheck("How many rounds would you like to play? ", 1)

    if games_played == 0:
        # Dice instructions
        gen_statement("?????? \t Choosing Dice... \t ??????","?")
        print("In a moment you will be asked to choose hwo many dice to play with\n\n"
              "Choose one die to create a race where six cars are involved.  To go forward, "
              "our virtual die is rolled and the car that matches the number moves forward"
              "one space.  The first car to move through the distance of the race wins.\n"
              "All cars have an equal chance of winning the race!\n\n"
              "Choose two dice to create a race involving 12 cars where to go forward.  In thi case the value of "
              "both dice are added together.  So if one die is a 3 and the other is a 4, car 7 "
              "will progress.  In this game, not all cars have an equal chance of winning.\n")
        games_played += 1       # ensures that full 'dice' instructions don't show up again

    var_num_dice = intcheck("How many dice should I use <1 or 2>? ", 1, 2)

    # Make cars...
    # Create car strings (ie: car 1 to 12)

    while rounds_played < num_rounds:
        all_cars = []

        if var_num_dice == 1:
            top_val = 7
        else:
            top_val = 13

        for item in range (1, top_val):
            # for single digit strings note the space between the # and the number
            if 1 <= item <= 9:
                name = "car # {} | ".format(item)
            else:
                name = "car #{} | ".format(item)
            all_cars.append(name)

        print()
        input("Press <enter> to Play...")

        print()
        gen_statement("-----  Round {} of {}  -----".format(rounds_played+1, num_rounds), "-")
        print()
        user_choice = intcheck("Choose a car <enter a number between {} and {}>: ".format(1, top_val-1), 1, top_val-1)

        print()

        # Race cars and find place holders...

        # Find Winner
        gen_statement("===  Racing .... <who will win>??  ===", "=")
        find_winner = race_them(all_cars, var_num_dice, [], "** 1st **", 0)

        round_score = 0
        if user_choice == int(find_winner[2]):
            print("****  Congratulations!  You came first. ****")
            round_score = 5
        print()
        input("Please press <enter>")
        print()

        # Find Second
        gen_statement("===  Racing .... <second place>??  ===", "=")
        find_second = race_them(all_cars, var_num_dice, find_winner[0], "* 2nd *", find_winner[1])
        if user_choice == int(find_second[2]):
            print("**** Well done, your car came in second ****")
            round_score = 3
        print()
        input("Please press <enter>")
        print()

        # Find Third Place winner
        gen_statement("===  Racing .... <who's on third>??  ===", "=")
        find_third = race_them(all_cars, var_num_dice, find_second[0], "* 3rd *", find_second[1])
        if user_choice == int(find_third[2]):
            print("**** Yay!  your car came in third ****")
            round_score = 1
        rounds_played += 1
        score += round_score

        # End of Round statements...
        print()
        if round_score == 0:
            feedback = "Sorry!  You did not place in this round"
            feedchar = "v"
        else:
            feedback = "Score for this round: {}".format(round_score)
            feedchar = "*"
        gen_statement("{} | \t Total Score: {}".format(feedback, score),feedchar)
        round_summary = "Round {}: {}".format(rounds_played, round_score)
        game_summary.append(round_summary)
        print()

    gen_statement("++++    End Game Results ++++", "+")
    # print Game Summary
    for item in game_summary:
        print(item, sep="")

    print("Total Score: {}".format(score))
    print("+" * len("++++    End Game Results ++++"))
    print()

    keep_going = input("Press <enter> to play again or any key to quit...")
    print()

print("Thanks for playing!")
